// FlagMaps.h: Line, Terminal and Address Caps flag maps

#include "FlagMap.h"

#define FLAGENTRY(flag) {flag, #flag}
#define FLAGENTRYEND() {0, 0}

FlagMap aAddressModes[] =
{
    FLAGENTRY(LINEADDRESSMODE_ADDRESSID),
    FLAGENTRY(LINEADDRESSMODE_DIALABLEADDR),
    FLAGENTRYEND()
};

FlagMap aBearerModes[] =
{
    FLAGENTRY(LINEBEARERMODE_VOICE),
    FLAGENTRY(LINEBEARERMODE_SPEECH),
    FLAGENTRY(LINEBEARERMODE_MULTIUSE),
    FLAGENTRY(LINEBEARERMODE_DATA),
    FLAGENTRY(LINEBEARERMODE_ALTSPEECHDATA),
    FLAGENTRY(LINEBEARERMODE_NONCALLSIGNALING),
    FLAGENTRY(LINEBEARERMODE_PASSTHROUGH),      // TAPI v1.4
    FLAGENTRY(LINEBEARERMODE_RESTRICTEDDATA),   // TAPI v2.0
    FLAGENTRYEND()
};

FlagMap aAnswerModes[] =
{
    FLAGENTRY(LINEANSWERMODE_NONE),
    FLAGENTRY(LINEANSWERMODE_DROP),
    FLAGENTRY(LINEANSWERMODE_HOLD),
    FLAGENTRYEND()
};

FlagMap aMediaModes[] =
{
    FLAGENTRY(LINEMEDIAMODE_UNKNOWN),
    FLAGENTRY(LINEMEDIAMODE_INTERACTIVEVOICE),
    FLAGENTRY(LINEMEDIAMODE_AUTOMATEDVOICE),
    FLAGENTRY(LINEMEDIAMODE_DATAMODEM),
    FLAGENTRY(LINEMEDIAMODE_G3FAX),
    FLAGENTRY(LINEMEDIAMODE_TDD),
    FLAGENTRY(LINEMEDIAMODE_G4FAX),
    FLAGENTRY(LINEMEDIAMODE_DIGITALDATA),
    FLAGENTRY(LINEMEDIAMODE_TELETEX),
    FLAGENTRY(LINEMEDIAMODE_VIDEOTEX),
    FLAGENTRY(LINEMEDIAMODE_TELEX),
    FLAGENTRY(LINEMEDIAMODE_MIXED),
    FLAGENTRY(LINEMEDIAMODE_ADSI),
    FLAGENTRY(LINEMEDIAMODE_VOICEVIEW),         // TAPI v1.4
    FLAGENTRY(LINEMEDIAMODE_VIDEO),             // TAPI v2.1
    FLAGENTRYEND()
};

FlagMap aToneModes[] =
{
    FLAGENTRY(LINETONEMODE_CUSTOM),
    FLAGENTRY(LINETONEMODE_RINGBACK),
    FLAGENTRY(LINETONEMODE_BUSY),
    FLAGENTRY(LINETONEMODE_BEEP),
    FLAGENTRY(LINETONEMODE_BILLING),
    FLAGENTRYEND()
};

FlagMap aDigitModes[] =
{
    FLAGENTRY(LINEDIGITMODE_PULSE),
    FLAGENTRY(LINEDIGITMODE_DTMF),
    FLAGENTRY(LINEDIGITMODE_DTMFEND),
    FLAGENTRYEND()
};

FlagMap aLineDevCapFlags[] =
{
    FLAGENTRY(LINEDEVCAPFLAGS_CROSSADDRCONF),
    FLAGENTRY(LINEDEVCAPFLAGS_HIGHLEVCOMP),
    FLAGENTRY(LINEDEVCAPFLAGS_LOWLEVCOMP),
    FLAGENTRY(LINEDEVCAPFLAGS_MEDIACONTROL),
    FLAGENTRY(LINEDEVCAPFLAGS_MULTIPLEADDR),
    FLAGENTRY(LINEDEVCAPFLAGS_CLOSEDROP),
    FLAGENTRY(LINEDEVCAPFLAGS_DIALBILLING),
    FLAGENTRY(LINEDEVCAPFLAGS_DIALQUIET),
    FLAGENTRY(LINEDEVCAPFLAGS_DIALDIALTONE),
#if (TAPI_CURRENT_VERSION >= 0x00030000)
    FLAGENTRY(LINEDEVCAPFLAGS_MSP),             // TAPI v3.0
    FLAGENTRY(LINEDEVCAPFLAGS_CALLHUB),         // TAPI v3.0
    FLAGENTRY(LINEDEVCAPFLAGS_CALLHUBTRACKING), // TAPI v3.0
    FLAGENTRY(LINEDEVCAPFLAGS_PRIVATEOBJECTS),  // TAPI v3.0
    FLAGENTRY(LINEDEVCAPFLAGS_LOCAL),           // TAPI v3.0
#endif
    FLAGENTRYEND()
};

FlagMap aLineDevStates[] =
{
    FLAGENTRY(LINEDEVSTATE_OTHER),
    FLAGENTRY(LINEDEVSTATE_RINGING),
    FLAGENTRY(LINEDEVSTATE_CONNECTED),
    FLAGENTRY(LINEDEVSTATE_DISCONNECTED),
    FLAGENTRY(LINEDEVSTATE_MSGWAITON),
    FLAGENTRY(LINEDEVSTATE_MSGWAITOFF),
    FLAGENTRY(LINEDEVSTATE_INSERVICE),
    FLAGENTRY(LINEDEVSTATE_OUTOFSERVICE),
    FLAGENTRY(LINEDEVSTATE_MAINTENANCE),
    FLAGENTRY(LINEDEVSTATE_OPEN),
    FLAGENTRY(LINEDEVSTATE_CLOSE),
    FLAGENTRY(LINEDEVSTATE_NUMCALLS),
    FLAGENTRY(LINEDEVSTATE_NUMCOMPLETIONS),
    FLAGENTRY(LINEDEVSTATE_TERMINALS),
    FLAGENTRY(LINEDEVSTATE_ROAMMODE),
    FLAGENTRY(LINEDEVSTATE_BATTERY),
    FLAGENTRY(LINEDEVSTATE_SIGNAL),
    FLAGENTRY(LINEDEVSTATE_DEVSPECIFIC),
    FLAGENTRY(LINEDEVSTATE_REINIT),
    FLAGENTRY(LINEDEVSTATE_LOCK),
    FLAGENTRY(LINEDEVSTATE_CAPSCHANGE),         // TAPI v1.4
    FLAGENTRY(LINEDEVSTATE_CONFIGCHANGE),       // TAPI v1.4
    FLAGENTRY(LINEDEVSTATE_TRANSLATECHANGE),    // TAPI v1.4
    FLAGENTRY(LINEDEVSTATE_COMPLCANCEL),        // TAPI v1.4
    FLAGENTRY(LINEDEVSTATE_REMOVED),            // TAPI v1.4
    FLAGENTRYEND()
};

FlagMap aLineFeatures[] =
{
    FLAGENTRY(LINEFEATURE_DEVSPECIFIC),
    FLAGENTRY(LINEFEATURE_DEVSPECIFICFEAT),
    FLAGENTRY(LINEFEATURE_FORWARD),
    FLAGENTRY(LINEFEATURE_MAKECALL),
    FLAGENTRY(LINEFEATURE_SETMEDIACONTROL),
    FLAGENTRY(LINEFEATURE_SETTERMINAL),
    FLAGENTRY(LINEFEATURE_SETDEVSTATUS),    // TAPI v2.0
    FLAGENTRY(LINEFEATURE_FORWARDFWD),      // TAPI v2.0
    FLAGENTRY(LINEFEATURE_FORWARDDND),      // TAPI v2.0
    FLAGENTRYEND()
};

FlagMap aTermModes[] =
{
    FLAGENTRY(LINETERMMODE_BUTTONS),
    FLAGENTRY(LINETERMMODE_LAMPS),
    FLAGENTRY(LINETERMMODE_DISPLAY),
    FLAGENTRY(LINETERMMODE_RINGER),
    FLAGENTRY(LINETERMMODE_HOOKSWITCH),
    FLAGENTRY(LINETERMMODE_MEDIATOLINE),
    FLAGENTRY(LINETERMMODE_MEDIAFROMLINE),
    FLAGENTRY(LINETERMMODE_MEDIABIDIRECT),
    FLAGENTRYEND()
};

FlagMap aAddressSharing[] = 
{
    FLAGENTRY(LINEADDRESSSHARING_PRIVATE),
    FLAGENTRY(LINEADDRESSSHARING_BRIDGEDEXCL),
    FLAGENTRY(LINEADDRESSSHARING_BRIDGEDNEW),
    FLAGENTRY(LINEADDRESSSHARING_BRIDGEDSHARED),
    FLAGENTRY(LINEADDRESSSHARING_MONITORED),
    FLAGENTRYEND()
};

FlagMap aAddressStates[] = 
{
    FLAGENTRY(LINEADDRESSSTATE_OTHER),
    FLAGENTRY(LINEADDRESSSTATE_DEVSPECIFIC),
    FLAGENTRY(LINEADDRESSSTATE_INUSEZERO),
    FLAGENTRY(LINEADDRESSSTATE_INUSEONE),
    FLAGENTRY(LINEADDRESSSTATE_INUSEMANY),
    FLAGENTRY(LINEADDRESSSTATE_NUMCALLS),
    FLAGENTRY(LINEADDRESSSTATE_FORWARD),
    FLAGENTRY(LINEADDRESSSTATE_TERMINALS),
    FLAGENTRY(LINEADDRESSSTATE_CAPSCHANGE), // TAPI v1.4
    FLAGENTRYEND()
};

FlagMap aCallInfoStates[] = 
{
    FLAGENTRY(LINECALLINFOSTATE_OTHER),
    FLAGENTRY(LINECALLINFOSTATE_DEVSPECIFIC),
    FLAGENTRY(LINECALLINFOSTATE_BEARERMODE),
    FLAGENTRY(LINECALLINFOSTATE_RATE),
    FLAGENTRY(LINECALLINFOSTATE_MEDIAMODE),
    FLAGENTRY(LINECALLINFOSTATE_APPSPECIFIC),
    FLAGENTRY(LINECALLINFOSTATE_CALLID),
    FLAGENTRY(LINECALLINFOSTATE_RELATEDCALLID),
    FLAGENTRY(LINECALLINFOSTATE_ORIGIN),
    FLAGENTRY(LINECALLINFOSTATE_REASON),
    FLAGENTRY(LINECALLINFOSTATE_COMPLETIONID),
    FLAGENTRY(LINECALLINFOSTATE_NUMOWNERINCR),
    FLAGENTRY(LINECALLINFOSTATE_NUMOWNERDECR),
    FLAGENTRY(LINECALLINFOSTATE_NUMMONITORS),
    FLAGENTRY(LINECALLINFOSTATE_TRUNK),
    FLAGENTRY(LINECALLINFOSTATE_CALLERID),
    FLAGENTRY(LINECALLINFOSTATE_CALLEDID),
    FLAGENTRY(LINECALLINFOSTATE_CONNECTEDID),
    FLAGENTRY(LINECALLINFOSTATE_REDIRECTIONID),
    FLAGENTRY(LINECALLINFOSTATE_REDIRECTINGID),
    FLAGENTRY(LINECALLINFOSTATE_DISPLAY),
    FLAGENTRY(LINECALLINFOSTATE_USERUSERINFO),
    FLAGENTRY(LINECALLINFOSTATE_HIGHLEVELCOMP),
    FLAGENTRY(LINECALLINFOSTATE_LOWLEVELCOMP),
    FLAGENTRY(LINECALLINFOSTATE_CHARGINGINFO),
    FLAGENTRY(LINECALLINFOSTATE_TERMINAL),
    FLAGENTRY(LINECALLINFOSTATE_DIALPARAMS),
    FLAGENTRY(LINECALLINFOSTATE_MONITORMODES),
    FLAGENTRY(LINECALLINFOSTATE_TREATMENT), // TAPI v2.0
    FLAGENTRY(LINECALLINFOSTATE_QOS),       // TAPI v2.0
    FLAGENTRY(LINECALLINFOSTATE_CALLDATA),  // TAPI v2.0
    FLAGENTRYEND()
};

FlagMap aCallPartyIDs[] = 
{
    FLAGENTRY(LINECALLPARTYID_BLOCKED),
    FLAGENTRY(LINECALLPARTYID_OUTOFAREA),
    FLAGENTRY(LINECALLPARTYID_NAME),
    FLAGENTRY(LINECALLPARTYID_ADDRESS),
    FLAGENTRY(LINECALLPARTYID_PARTIAL),
    FLAGENTRY(LINECALLPARTYID_UNKNOWN),
    FLAGENTRY(LINECALLPARTYID_UNAVAIL),
    FLAGENTRYEND()
};

FlagMap*    aCallerIDFlags = aCallPartyIDs;
FlagMap*    aCalledIDFlags = aCallPartyIDs;
FlagMap*    aConnectedIDFlags = aCallPartyIDs;
FlagMap*    aRedirectionIDFlags = aCallPartyIDs;
FlagMap*    aRedirectingIDFlags = aCallPartyIDs;

FlagMap aCallStates[] = 
{
    FLAGENTRY(LINECALLSTATE_IDLE),
    FLAGENTRY(LINECALLSTATE_OFFERING),
    FLAGENTRY(LINECALLSTATE_ACCEPTED),
    FLAGENTRY(LINECALLSTATE_DIALTONE),
    FLAGENTRY(LINECALLSTATE_DIALING),
    FLAGENTRY(LINECALLSTATE_RINGBACK),
    FLAGENTRY(LINECALLSTATE_BUSY),
    FLAGENTRY(LINECALLSTATE_SPECIALINFO),
    FLAGENTRY(LINECALLSTATE_CONNECTED),
    FLAGENTRY(LINECALLSTATE_PROCEEDING),
    FLAGENTRY(LINECALLSTATE_ONHOLD),
    FLAGENTRY(LINECALLSTATE_CONFERENCED),
    FLAGENTRY(LINECALLSTATE_ONHOLDPENDCONF),
    FLAGENTRY(LINECALLSTATE_ONHOLDPENDTRANSFER),
    FLAGENTRY(LINECALLSTATE_DISCONNECTED),
    FLAGENTRY(LINECALLSTATE_UNKNOWN),
    FLAGENTRYEND()
};

FlagMap aDialToneModes[] = 
{
    FLAGENTRY(LINEDIALTONEMODE_NORMAL),
    FLAGENTRY(LINEDIALTONEMODE_SPECIAL),
    FLAGENTRY(LINEDIALTONEMODE_INTERNAL),
    FLAGENTRY(LINEDIALTONEMODE_EXTERNAL),
    FLAGENTRY(LINEDIALTONEMODE_UNKNOWN),
    FLAGENTRY(LINEDIALTONEMODE_UNAVAIL),
    FLAGENTRYEND()
};

FlagMap aBusyModes[] = 
{
    FLAGENTRY(LINEBUSYMODE_STATION),
    FLAGENTRY(LINEBUSYMODE_TRUNK),
    FLAGENTRY(LINEBUSYMODE_UNKNOWN),
    FLAGENTRY(LINEBUSYMODE_UNAVAIL),
    FLAGENTRYEND()
};

FlagMap aSpecialInfos[] = 
{
    FLAGENTRY(LINESPECIALINFO_NOCIRCUIT),
    FLAGENTRY(LINESPECIALINFO_CUSTIRREG),
    FLAGENTRY(LINESPECIALINFO_REORDER),
    FLAGENTRY(LINESPECIALINFO_UNKNOWN),
    FLAGENTRY(LINESPECIALINFO_UNAVAIL),
    FLAGENTRYEND()
};

FlagMap aDisconnectModes[] = 
{
    FLAGENTRY(LINEDISCONNECTMODE_NORMAL),
    FLAGENTRY(LINEDISCONNECTMODE_UNKNOWN),
    FLAGENTRY(LINEDISCONNECTMODE_REJECT),
    FLAGENTRY(LINEDISCONNECTMODE_PICKUP),
    FLAGENTRY(LINEDISCONNECTMODE_FORWARDED),
    FLAGENTRY(LINEDISCONNECTMODE_BUSY),
    FLAGENTRY(LINEDISCONNECTMODE_NOANSWER),
    FLAGENTRY(LINEDISCONNECTMODE_BADADDRESS),
    FLAGENTRY(LINEDISCONNECTMODE_UNREACHABLE),
    FLAGENTRY(LINEDISCONNECTMODE_CONGESTION),
    FLAGENTRY(LINEDISCONNECTMODE_INCOMPATIBLE),
    FLAGENTRY(LINEDISCONNECTMODE_UNAVAIL),
    FLAGENTRY(LINEDISCONNECTMODE_NODIALTONE),
    FLAGENTRY(LINEDISCONNECTMODE_NUMBERCHANGED),    // TAPI v2.0
    FLAGENTRY(LINEDISCONNECTMODE_OUTOFORDER),       // TAPI v2.0
    FLAGENTRY(LINEDISCONNECTMODE_TEMPFAILURE),      // TAPI v2.0
    FLAGENTRY(LINEDISCONNECTMODE_QOSUNAVAIL),       // TAPI v2.0
    FLAGENTRY(LINEDISCONNECTMODE_BLOCKED),          // TAPI v2.0
    FLAGENTRY(LINEDISCONNECTMODE_DONOTDISTURB),     // TAPI v2.0
    FLAGENTRY(LINEDISCONNECTMODE_CANCELLED),        // TAPI v2.0
    FLAGENTRYEND()
};

FlagMap aAddrCapFlags[] = 
{
    FLAGENTRY(LINEADDRCAPFLAGS_FWDNUMRINGS),
    FLAGENTRY(LINEADDRCAPFLAGS_PICKUPGROUPID),
    FLAGENTRY(LINEADDRCAPFLAGS_SECURE),
    FLAGENTRY(LINEADDRCAPFLAGS_BLOCKIDDEFAULT),
    FLAGENTRY(LINEADDRCAPFLAGS_BLOCKIDOVERRIDE),
    FLAGENTRY(LINEADDRCAPFLAGS_DIALED),
    FLAGENTRY(LINEADDRCAPFLAGS_ORIGOFFHOOK),
    FLAGENTRY(LINEADDRCAPFLAGS_DESTOFFHOOK),
    FLAGENTRY(LINEADDRCAPFLAGS_FWDCONSULT),
    FLAGENTRY(LINEADDRCAPFLAGS_SETUPCONFNULL),
    FLAGENTRY(LINEADDRCAPFLAGS_AUTORECONNECT),
    FLAGENTRY(LINEADDRCAPFLAGS_COMPLETIONID),
    FLAGENTRY(LINEADDRCAPFLAGS_TRANSFERHELD),
    FLAGENTRY(LINEADDRCAPFLAGS_TRANSFERMAKE),
    FLAGENTRY(LINEADDRCAPFLAGS_CONFERENCEHELD),
    FLAGENTRY(LINEADDRCAPFLAGS_CONFERENCEMAKE),
    FLAGENTRY(LINEADDRCAPFLAGS_PARTIALDIAL),
    FLAGENTRY(LINEADDRCAPFLAGS_FWDSTATUSVALID),
    FLAGENTRY(LINEADDRCAPFLAGS_FWDINTEXTADDR),
    FLAGENTRY(LINEADDRCAPFLAGS_FWDBUSYNAADDR),
    FLAGENTRY(LINEADDRCAPFLAGS_ACCEPTTOALERT),
    FLAGENTRY(LINEADDRCAPFLAGS_CONFDROP),
    FLAGENTRY(LINEADDRCAPFLAGS_PICKUPCALLWAIT),
    FLAGENTRY(LINEADDRCAPFLAGS_PREDICTIVEDIALER),           // TAPI v2.0
    FLAGENTRY(LINEADDRCAPFLAGS_QUEUE),                      // TAPI v2.0
    FLAGENTRY(LINEADDRCAPFLAGS_ROUTEPOINT),                 // TAPI v2.0
    FLAGENTRY(LINEADDRCAPFLAGS_HOLDMAKESNEW),               // TAPI v2.0
    FLAGENTRY(LINEADDRCAPFLAGS_NOINTERNALCALLS),            // TAPI v2.0
    FLAGENTRY(LINEADDRCAPFLAGS_NOEXTERNALCALLS),            // TAPI v2.0
    FLAGENTRY(LINEADDRCAPFLAGS_SETCALLINGID),               // TAPI v2.0

    // VC6 headers don't have this
#ifdef LINEADDRCAPFLAGS_ACDGROUP
    FLAGENTRY(LINEADDRCAPFLAGS_ACDGROUP),                   // TAPI v2.2
#endif
#if (TAPI_CURRENT_VERSION >= 0x00030000)
    FLAGENTRY(LINEADDRCAPFLAGS_NOPSTNADDRESSTRANSLATION),   // TAPI v3.0
#endif
    FLAGENTRYEND()
};

FlagMap aCallFeatures[] = 
{
    FLAGENTRY(LINECALLFEATURE_ACCEPT),
    FLAGENTRY(LINECALLFEATURE_ADDTOCONF),
    FLAGENTRY(LINECALLFEATURE_ANSWER),
    FLAGENTRY(LINECALLFEATURE_BLINDTRANSFER),
    FLAGENTRY(LINECALLFEATURE_COMPLETECALL),
    FLAGENTRY(LINECALLFEATURE_COMPLETETRANSF),
    FLAGENTRY(LINECALLFEATURE_DIAL),
    FLAGENTRY(LINECALLFEATURE_DROP),
    FLAGENTRY(LINECALLFEATURE_GATHERDIGITS),
    FLAGENTRY(LINECALLFEATURE_GENERATEDIGITS),
    FLAGENTRY(LINECALLFEATURE_GENERATETONE),
    FLAGENTRY(LINECALLFEATURE_HOLD),
    FLAGENTRY(LINECALLFEATURE_MONITORDIGITS),
    FLAGENTRY(LINECALLFEATURE_MONITORMEDIA),
    FLAGENTRY(LINECALLFEATURE_MONITORTONES),
    FLAGENTRY(LINECALLFEATURE_PARK),
    FLAGENTRY(LINECALLFEATURE_PREPAREADDCONF),
    FLAGENTRY(LINECALLFEATURE_REDIRECT),
    FLAGENTRY(LINECALLFEATURE_REMOVEFROMCONF),
    FLAGENTRY(LINECALLFEATURE_SECURECALL),
    FLAGENTRY(LINECALLFEATURE_SENDUSERUSER),
    FLAGENTRY(LINECALLFEATURE_SETCALLPARAMS),
    FLAGENTRY(LINECALLFEATURE_SETMEDIACONTROL),
    FLAGENTRY(LINECALLFEATURE_SETTERMINAL),
    FLAGENTRY(LINECALLFEATURE_SETUPCONF),
    FLAGENTRY(LINECALLFEATURE_SETUPTRANSFER),
    FLAGENTRY(LINECALLFEATURE_SWAPHOLD),
    FLAGENTRY(LINECALLFEATURE_UNHOLD),
    FLAGENTRY(LINECALLFEATURE_RELEASEUSERUSERINFO), // TAPI v1.4
    FLAGENTRY(LINECALLFEATURE_SETTREATMENT),        // TAPI v2.0
    FLAGENTRY(LINECALLFEATURE_SETQOS),              // TAPI v2.0
    FLAGENTRY(LINECALLFEATURE_SETCALLDATA),         // TAPI v2.0
    FLAGENTRYEND()
};

FlagMap aCallFeatures2[] = 
{
    FLAGENTRY(LINECALLFEATURE2_NOHOLDCONFERENCE),   // TAPI v2.0
    FLAGENTRY(LINECALLFEATURE2_ONESTEPTRANSFER),    // TAPI v2.0
    FLAGENTRY(LINECALLFEATURE2_COMPLCAMPON),        // TAPI v2.0
    FLAGENTRY(LINECALLFEATURE2_COMPLCALLBACK),      // TAPI v2.0
    FLAGENTRY(LINECALLFEATURE2_COMPLINTRUDE),       // TAPI v2.0
    FLAGENTRY(LINECALLFEATURE2_COMPLMESSAGE),       // TAPI v2.0
    FLAGENTRY(LINECALLFEATURE2_TRANSFERNORM),       // TAPI v2.0
    FLAGENTRY(LINECALLFEATURE2_TRANSFERCONF),       // TAPI v2.0
    FLAGENTRY(LINECALLFEATURE2_PARKDIRECT),         // TAPI v2.0
    FLAGENTRY(LINECALLFEATURE2_PARKNONDIRECT),      // TAPI v2.0
    FLAGENTRYEND()
};

FlagMap aTransferModes[] = 
{
    FLAGENTRY(LINETRANSFERMODE_TRANSFER),
    FLAGENTRY(LINETRANSFERMODE_CONFERENCE),
    FLAGENTRYEND()
};

FlagMap aParkModes[] = 
{
    FLAGENTRY(LINEPARKMODE_DIRECTED),
    FLAGENTRY(LINEPARKMODE_NONDIRECTED),
    FLAGENTRYEND()
};

FlagMap aForwardModes[] = 
{
    FLAGENTRY(LINEFORWARDMODE_UNCOND),
    FLAGENTRY(LINEFORWARDMODE_UNCONDINTERNAL),
    FLAGENTRY(LINEFORWARDMODE_UNCONDEXTERNAL),
    FLAGENTRY(LINEFORWARDMODE_UNCONDSPECIFIC),
    FLAGENTRY(LINEFORWARDMODE_BUSY),
    FLAGENTRY(LINEFORWARDMODE_BUSYINTERNAL),
    FLAGENTRY(LINEFORWARDMODE_BUSYEXTERNAL),
    FLAGENTRY(LINEFORWARDMODE_BUSYSPECIFIC),
    FLAGENTRY(LINEFORWARDMODE_NOANSW),
    FLAGENTRY(LINEFORWARDMODE_NOANSWINTERNAL),
    FLAGENTRY(LINEFORWARDMODE_NOANSWEXTERNAL),
    FLAGENTRY(LINEFORWARDMODE_NOANSWSPECIFIC),
    FLAGENTRY(LINEFORWARDMODE_BUSYNA),
    FLAGENTRY(LINEFORWARDMODE_BUSYNAINTERNAL),
    FLAGENTRY(LINEFORWARDMODE_BUSYNAEXTERNAL),
    FLAGENTRY(LINEFORWARDMODE_BUSYNASPECIFIC),
    FLAGENTRY(LINEFORWARDMODE_UNKNOWN),         // TAPI v1.4
    FLAGENTRY(LINEFORWARDMODE_UNAVAIL),         // TAPI v1.4
    FLAGENTRYEND()
};

FlagMap aCallCompletionConds[] = 
{
    FLAGENTRY(LINECALLCOMPLCOND_BUSY),
    FLAGENTRY(LINECALLCOMPLCOND_NOANSWER),
    FLAGENTRYEND()
};

FlagMap aCallCompletionModes[] = 
{
    FLAGENTRY(LINECALLCOMPLMODE_CAMPON),
    FLAGENTRY(LINECALLCOMPLMODE_CALLBACK),
    FLAGENTRY(LINECALLCOMPLMODE_INTRUDE),
    FLAGENTRY(LINECALLCOMPLMODE_MESSAGE),
    FLAGENTRYEND()
};

FlagMap aAddressFeatures[] = 
{
    FLAGENTRY(LINEADDRFEATURE_FORWARD),
    FLAGENTRY(LINEADDRFEATURE_MAKECALL),
    FLAGENTRY(LINEADDRFEATURE_PICKUP),
    FLAGENTRY(LINEADDRFEATURE_SETMEDIACONTROL),
    FLAGENTRY(LINEADDRFEATURE_SETTERMINAL),
    FLAGENTRY(LINEADDRFEATURE_SETUPCONF),
    FLAGENTRY(LINEADDRFEATURE_UNCOMPLETECALL),
    FLAGENTRY(LINEADDRFEATURE_UNPARK),
    FLAGENTRY(LINEADDRFEATURE_PICKUPHELD),      // TAPI v2.0
    FLAGENTRY(LINEADDRFEATURE_PICKUPGROUP),     // TAPI v2.0
    FLAGENTRY(LINEADDRFEATURE_PICKUPDIRECT),    // TAPI v2.0
    FLAGENTRY(LINEADDRFEATURE_PICKUPWAITING),   // TAPI v2.0
    FLAGENTRY(LINEADDRFEATURE_FORWARDFWD),      // TAPI v2.0
    FLAGENTRY(LINEADDRFEATURE_FORWARDDND),      // TAPI v2.0
    FLAGENTRYEND()
};

FlagMap aTermDevs[] =
{
    FLAGENTRY(LINETERMDEV_PHONE),
    FLAGENTRY(LINETERMDEV_HEADSET),
    FLAGENTRY(LINETERMDEV_SPEAKER),
    FLAGENTRYEND()
};

FlagMap aTermSharing[] =
{
    FLAGENTRY(LINETERMSHARING_PRIVATE),
    FLAGENTRY(LINETERMSHARING_SHAREDEXCL),
    FLAGENTRY(LINETERMSHARING_SHAREDCONF),
    FLAGENTRYEND()
};

FlagMap aRemoveFromConf[] = 
{
    FLAGENTRY(LINEREMOVEFROMCONF_NONE),
    FLAGENTRY(LINEREMOVEFROMCONF_LAST),
    FLAGENTRY(LINEREMOVEFROMCONF_ANY),
    FLAGENTRYEND()
};

FlagMap aLocationOptions[] =
{
    FLAGENTRY(LINELOCATIONOPTION_PULSEDIAL),
    FLAGENTRYEND()
};

FlagMap aCardOptions[] =
{
    FLAGENTRY(LINECARDOPTION_PREDEFINED),
    FLAGENTRY(LINECARDOPTION_HIDDEN),
    FLAGENTRYEND()
};

FlagMap aPhoneStates[] =
{
    FLAGENTRY(PHONESTATE_OTHER),
    FLAGENTRY(PHONESTATE_CONNECTED),
    FLAGENTRY(PHONESTATE_DISCONNECTED),
    FLAGENTRY(PHONESTATE_OWNER),
    FLAGENTRY(PHONESTATE_MONITORS),
    FLAGENTRY(PHONESTATE_DISPLAY),
    FLAGENTRY(PHONESTATE_LAMP),
    FLAGENTRY(PHONESTATE_RINGMODE),
    FLAGENTRY(PHONESTATE_RINGVOLUME),
    FLAGENTRY(PHONESTATE_HANDSETHOOKSWITCH),
    FLAGENTRY(PHONESTATE_HANDSETVOLUME),
    FLAGENTRY(PHONESTATE_HANDSETGAIN),
    FLAGENTRY(PHONESTATE_SPEAKERHOOKSWITCH),
    FLAGENTRY(PHONESTATE_SPEAKERVOLUME),
    FLAGENTRY(PHONESTATE_SPEAKERGAIN),
    FLAGENTRY(PHONESTATE_HEADSETHOOKSWITCH),
    FLAGENTRY(PHONESTATE_HEADSETVOLUME),
    FLAGENTRY(PHONESTATE_HEADSETGAIN),
    FLAGENTRY(PHONESTATE_SUSPEND),
    FLAGENTRY(PHONESTATE_RESUME),
    FLAGENTRY(PHONESTATE_DEVSPECIFIC),
    FLAGENTRY(PHONESTATE_REINIT),
    FLAGENTRY(PHONESTATE_CAPSCHANGE),   // TAPI v1.4
    FLAGENTRY(PHONESTATE_REMOVED),      // TAPI v1.4
    FLAGENTRYEND()
};

FlagMap aHookSwitchDevs[] =
{
    FLAGENTRY(PHONEHOOKSWITCHDEV_HANDSET),
    FLAGENTRY(PHONEHOOKSWITCHDEV_SPEAKER),
    FLAGENTRY(PHONEHOOKSWITCHDEV_HEADSET),
    FLAGENTRYEND()
};

FlagMap aHookSwitchModes[] =
{
    FLAGENTRY(PHONEHOOKSWITCHMODE_ONHOOK),
    FLAGENTRY(PHONEHOOKSWITCHMODE_MIC),
    FLAGENTRY(PHONEHOOKSWITCHMODE_SPEAKER),
    FLAGENTRY(PHONEHOOKSWITCHMODE_MICSPEAKER),
    FLAGENTRY(PHONEHOOKSWITCHMODE_UNKNOWN),
    FLAGENTRYEND()
};

FlagMap aButtonModes[] =
{
    FLAGENTRY(PHONEBUTTONMODE_DUMMY),
    FLAGENTRY(PHONEBUTTONMODE_CALL),
    FLAGENTRY(PHONEBUTTONMODE_FEATURE),
    FLAGENTRY(PHONEBUTTONMODE_KEYPAD),
    FLAGENTRY(PHONEBUTTONMODE_LOCAL),
    FLAGENTRY(PHONEBUTTONMODE_DISPLAY),
    FLAGENTRYEND()
};

FlagMap aButtonFunctions[] =
{
    FLAGENTRY(PHONEBUTTONFUNCTION_UNKNOWN),
    FLAGENTRY(PHONEBUTTONFUNCTION_CONFERENCE),
    FLAGENTRY(PHONEBUTTONFUNCTION_TRANSFER),
    FLAGENTRY(PHONEBUTTONFUNCTION_DROP),
    FLAGENTRY(PHONEBUTTONFUNCTION_HOLD),
    FLAGENTRY(PHONEBUTTONFUNCTION_RECALL),
    FLAGENTRY(PHONEBUTTONFUNCTION_DISCONNECT),
    FLAGENTRY(PHONEBUTTONFUNCTION_CONNECT),
    FLAGENTRY(PHONEBUTTONFUNCTION_MSGWAITON),
    FLAGENTRY(PHONEBUTTONFUNCTION_MSGWAITOFF),
    FLAGENTRY(PHONEBUTTONFUNCTION_SELECTRING),
    FLAGENTRY(PHONEBUTTONFUNCTION_ABBREVDIAL),
    FLAGENTRY(PHONEBUTTONFUNCTION_FORWARD),
    FLAGENTRY(PHONEBUTTONFUNCTION_PICKUP),
    FLAGENTRY(PHONEBUTTONFUNCTION_RINGAGAIN),
    FLAGENTRY(PHONEBUTTONFUNCTION_PARK),
    FLAGENTRY(PHONEBUTTONFUNCTION_REJECT),
    FLAGENTRY(PHONEBUTTONFUNCTION_REDIRECT),
    FLAGENTRY(PHONEBUTTONFUNCTION_MUTE),
    FLAGENTRY(PHONEBUTTONFUNCTION_VOLUMEUP),
    FLAGENTRY(PHONEBUTTONFUNCTION_VOLUMEDOWN),
    FLAGENTRY(PHONEBUTTONFUNCTION_SPEAKERON),
    FLAGENTRY(PHONEBUTTONFUNCTION_SPEAKEROFF),
    FLAGENTRY(PHONEBUTTONFUNCTION_FLASH),
    FLAGENTRY(PHONEBUTTONFUNCTION_DATAON),
    FLAGENTRY(PHONEBUTTONFUNCTION_DATAOFF),
    FLAGENTRY(PHONEBUTTONFUNCTION_DONOTDISTURB),
    FLAGENTRY(PHONEBUTTONFUNCTION_INTERCOM),
    FLAGENTRY(PHONEBUTTONFUNCTION_BRIDGEDAPP),
    FLAGENTRY(PHONEBUTTONFUNCTION_BUSY),
    FLAGENTRY(PHONEBUTTONFUNCTION_CALLAPP),
    FLAGENTRY(PHONEBUTTONFUNCTION_DATETIME),
    FLAGENTRY(PHONEBUTTONFUNCTION_DIRECTORY),
    FLAGENTRY(PHONEBUTTONFUNCTION_COVER),
    FLAGENTRY(PHONEBUTTONFUNCTION_CALLID),
    FLAGENTRY(PHONEBUTTONFUNCTION_LASTNUM),
    FLAGENTRY(PHONEBUTTONFUNCTION_NIGHTSRV),
    FLAGENTRY(PHONEBUTTONFUNCTION_SENDCALLS),
    FLAGENTRY(PHONEBUTTONFUNCTION_MSGINDICATOR),
    FLAGENTRY(PHONEBUTTONFUNCTION_REPDIAL),
    FLAGENTRY(PHONEBUTTONFUNCTION_SETREPDIAL),
    FLAGENTRY(PHONEBUTTONFUNCTION_SYSTEMSPEED),
    FLAGENTRY(PHONEBUTTONFUNCTION_STATIONSPEED),
    FLAGENTRY(PHONEBUTTONFUNCTION_CAMPON),
    FLAGENTRY(PHONEBUTTONFUNCTION_SAVEREPEAT),
    FLAGENTRY(PHONEBUTTONFUNCTION_QUEUECALL),
    FLAGENTRY(PHONEBUTTONFUNCTION_NONE),
#if (TAPI_CURRENT_VERSION >= 0x00030001)
    FLAGENTRY(PHONEBUTTONFUNCTION_SEND),        // TAPI v3.1
#endif
    FLAGENTRYEND()
};

FlagMap aLampModes[] =
{
    FLAGENTRY(PHONELAMPMODE_DUMMY),
    FLAGENTRY(PHONELAMPMODE_OFF),
    FLAGENTRY(PHONELAMPMODE_STEADY),
    FLAGENTRY(PHONELAMPMODE_WINK),
    FLAGENTRY(PHONELAMPMODE_FLASH),
    FLAGENTRY(PHONELAMPMODE_FLUTTER),
    FLAGENTRY(PHONELAMPMODE_BROKENFLUTTER),
    FLAGENTRY(PHONELAMPMODE_UNKNOWN),
    FLAGENTRYEND()
};

FlagMap aCallParamFlags[] =
{
    FLAGENTRY(LINECALLPARAMFLAGS_SECURE),
    FLAGENTRY(LINECALLPARAMFLAGS_IDLE),
    FLAGENTRY(LINECALLPARAMFLAGS_BLOCKID),
    FLAGENTRY(LINECALLPARAMFLAGS_ORIGOFFHOOK),
    FLAGENTRY(LINECALLPARAMFLAGS_DESTOFFHOOK),
    FLAGENTRY(LINECALLPARAMFLAGS_NOHOLDCONFERENCE), // TAPI v2.0
    FLAGENTRY(LINECALLPARAMFLAGS_PREDICTIVEDIAL),   // TAPI v2.0
    FLAGENTRY(LINECALLPARAMFLAGS_ONESTEPTRANSFER),  // TAPI v2.0
    FLAGENTRYEND()
};

FlagMap aCallOrigins[] =
{
    FLAGENTRY(LINECALLORIGIN_OUTBOUND),
    FLAGENTRY(LINECALLORIGIN_INTERNAL),
    FLAGENTRY(LINECALLORIGIN_EXTERNAL),
    FLAGENTRY(LINECALLORIGIN_UNKNOWN),
    FLAGENTRY(LINECALLORIGIN_UNAVAIL),
    FLAGENTRY(LINECALLORIGIN_CONFERENCE),
    FLAGENTRY(LINECALLORIGIN_INBOUND),  // TAPI v1.4
    FLAGENTRYEND()
};

FlagMap aCallReasons[] =
{
    FLAGENTRY(LINECALLREASON_DIRECT),
    FLAGENTRY(LINECALLREASON_FWDBUSY),
    FLAGENTRY(LINECALLREASON_FWDNOANSWER),
    FLAGENTRY(LINECALLREASON_FWDUNCOND),
    FLAGENTRY(LINECALLREASON_PICKUP),
    FLAGENTRY(LINECALLREASON_UNPARK),
    FLAGENTRY(LINECALLREASON_REDIRECT),
    FLAGENTRY(LINECALLREASON_CALLCOMPLETION),
    FLAGENTRY(LINECALLREASON_TRANSFER),
    FLAGENTRY(LINECALLREASON_REMINDER),
    FLAGENTRY(LINECALLREASON_UNKNOWN),
    FLAGENTRY(LINECALLREASON_UNAVAIL),
    FLAGENTRY(LINECALLREASON_INTRUDE),      // TAPI v1.4
    FLAGENTRY(LINECALLREASON_PARKED),       // TAPI v1.4
    FLAGENTRY(LINECALLREASON_CAMPEDON),     // TAPI v2.0
    FLAGENTRY(LINECALLREASON_ROUTEREQUEST), // TAPI v2.0
    FLAGENTRYEND()
};

FlagMap aAddressTypes[] =
{
#if (TAPI_CURRENT_VERSION >= 0x00030000)
    FLAGENTRY(LINEADDRESSTYPE_PHONENUMBER), // TAPI v3.0
    FLAGENTRY(LINEADDRESSTYPE_SDP),         // TAPI v3.0
    FLAGENTRY(LINEADDRESSTYPE_EMAILNAME),   // TAPI v3.0
    FLAGENTRY(LINEADDRESSTYPE_DOMAINNAME),  // TAPI v3.0
    FLAGENTRY(LINEADDRESSTYPE_IPADDRESS),   // TAPI v3.0
#endif
    FLAGENTRYEND()
};

FlagMap aConnectedModes[] =
{
    FLAGENTRY(LINECONNECTEDMODE_ACTIVE),        // TAPI v1.4
    FLAGENTRY(LINECONNECTEDMODE_INACTIVE),      // TAPI v1.4
    FLAGENTRY(LINECONNECTEDMODE_ACTIVEHELD),    // TAPI v2.0
    FLAGENTRY(LINECONNECTEDMODE_INACTIVEHELD),  // TAPI v2.0
    FLAGENTRY(LINECONNECTEDMODE_CONFIRMED),     // TAPI v2.0
    FLAGENTRYEND()
};

FlagMap aOfferingModes[] =
{
    FLAGENTRY(LINEOFFERINGMODE_ACTIVE),     // TAPI v1.4
    FLAGENTRY(LINEOFFERINGMODE_INACTIVE),   // TAPI v1.4
    FLAGENTRYEND()
};

FlagMap aCallPrivileges[] =
{
    FLAGENTRY(LINECALLPRIVILEGE_NONE),
    FLAGENTRY(LINECALLPRIVILEGE_MONITOR),
    FLAGENTRY(LINECALLPRIVILEGE_OWNER),
    FLAGENTRYEND()
};

FlagMap aCallHubTracking[] =
{
#if (TAPI_CURRENT_VERSION >= 0x00030000)
    FLAGENTRY(LINECALLHUBTRACKING_NONE),            // TAPI v3.0
    FLAGENTRY(LINECALLHUBTRACKING_PROVIDERLEVEL),   // TAPI v3.0
    FLAGENTRY(LINECALLHUBTRACKING_ALLCALLS),        // TAPI v3.0
#endif
    FLAGENTRYEND()
};


#undef FLAGENTRY
#undef FLAGENTRYEND

